@@ -1,11 +1,7 @@
 /*
-**  Copyright (c) 2012, The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2012, The Trusted Domain Project.  All rights reserved.
 */
 
-#ifndef lint
-static char dkim_report_c_id[] = "@(#)$Id$";
-#endif /* !lint */
-
 #include "build-config.h"
 
 /* system includes */
@@ -101,6 +97,11 @@ dkim_repinfo(DKIM *dkim, DKIM_SIGINFO *sig, struct timeval *timeout,
 
 	/* XXX -- add QUERY_CACHE support here */
 
+	if (lib->dkiml_dns_service == NULL &&
+	    lib->dkiml_dns_init != NULL &&
+	    lib->dkiml_dns_init(&lib->dkiml_dns_service) != 0)
+		return DKIM_STAT_CANTVRFY;
+
 	/* send it */
 	anslen = sizeof ansbuf;
 	status = lib->dkiml_dns_start(lib->dkiml_dns_service, T_TXT,
