@@ -1,11 +1,7 @@
 /*
-**  Copyright (c) 2010, 2011, The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2010-2012, The Trusted Domain Project.  All rights reserved.
 */
 
-#ifndef lint
-static char rbl_c_id[] = "$Id$";
-#endif /* !lint */
-
 #include "build-config.h"
 
 /* system includes */
@@ -54,12 +50,19 @@ struct rbl_handle
 	void			(*rbl_free) (void *closure, void *p);
 	void			(*rbl_dns_callback) (const void *context);
 	void *			rbl_dns_service;
+	int			(*rbl_dns_init) (void **srv);
+	void			(*rbl_dns_close) (void *srv);
 	int			(*rbl_dns_start) (void *srv, int type,
 				                  unsigned char *query,
 				                  unsigned char *buf,
 				                  size_t buflen,
 				                  void **qh);
 	int			(*rbl_dns_cancel) (void *srv, void *qh);
+	int			(*rbl_dns_config) (void *srv,
+				                   const char *config);
+	int			(*rbl_dns_trustanchor) (void *srv,
+				                        const char *trust);
+	int			(*rbl_dns_setns) (void *srv, const char *ns);
 	int			(*rbl_dns_waitreply) (void *srv,
 				                      void *qh,
 				                      struct timeval *to,
@@ -231,6 +234,110 @@ rbl_res_waitreply(void *srv, void *qh, struct timeval *to, size_t *bytes,
 	return RBL_DNS_SUCCESS;
 }
 
+/*
+**  RBL_RES_SETNS -- set nameserver list
+**
+**  Parameters:
+**  	srv -- service handle
+**  	nslist -- nameserver list, as a string
+**
+**  Return value:
+**  	0 -- success
+**  	!0 -- error
+*/
+
+int
+rbl_res_nslist(void *srv, const char *nslist)
+{
+#ifdef HAVE_RES_SETSERVERS
+	int nscount = 0;
+	char *tmp;
+	char *ns;
+	char *last = NULL;
+	struct sockaddr_in in;
+# ifdef AF_INET6
+	struct sockaddr_in6 in6;
+# endif /* AF_INET6 */
+	struct state *res;
+	res_sockaddr_union nses[MAXNS];
+
+	assert(srv != NULL);
+	assert(nslist != NULL);
+
+	memset(nses, '\0', sizeof nses);
+
+	tmp = strdup(nslist);
+	if (tmp == NULL)
+		return -1;
+
+	for (ns = strtok_r(tmp, ",", &last);
+	     ns != NULL && nscount < MAXNS;
+	     ns = strtok_r(NULL, ",", &last)
+	{
+		memset(&in, '\0', sizeof in);
+# ifdef AF_INET6
+		memset(&in6, '\0', sizeof in6);
+# endif /* AF_INET6 */
+
+		if (inet_pton(AF_INET, ns, (struct in_addr *) &in.sin_addr,
+		              sizeof in.sin_addr) == 1)
+		{
+			in.sin_family= AF_INET;
+			in.sin_port = htons(DNSPORT);
+			memcpy(&nses[nscount].sin, &in,
+			       sizeof nses[nscount].sin);
+			nscount++;
+		}
+# ifdef AF_INET6
+		else if (inet_pton(AF_INET6, ns,
+		                   (struct in6_addr *) &in6.sin6_addr,
+		                   sizeof in6.sin6_addr) == 1)
+		{
+			in6.sin6_family= AF_INET6;
+			in6.sin6_port = htons(DNSPORT);
+			memcpy(&nses[nscount].sin6, &in6,
+			       sizeof nses[nscount].sin6);
+			nscount++;
+		}
+# endif /* AF_INET6 */
+		else
+		{
+			free(tmp);
+			return -1;
+		}
+	}
+
+	res = srv;
+	res_setservers(res, nses, nscount);
+
+	free(tmp);
+#endif /* HAVE_RES_SETSERVERS */
+}
+
+/*
+**  RBL_RES_CLOSE -- shut down the resolver
+**
+**  Parameters:
+**  	srv -- service handle
+**
+**  Return value:
+**  	None.
+*/
+
+void
+rbl_res_close(void *srv)
+{
+#ifdef HAVE_RES_NINIT
+	struct state *res;
+
+	res = srv;
+
+	res_nclose(res);
+
+	free(res);
+#endif /* HAVE_RES_NINIT */
+}
+
 /*
 **  RBL_INIT -- initialize an RBL handle
 **
@@ -271,6 +378,8 @@ rbl_init(void *(*caller_mallocf)(void *closure, size_t nbytes),
 	new->rbl_dns_start = rbl_res_query;
 	new->rbl_dns_waitreply = rbl_res_waitreply;
 	new->rbl_dns_cancel = rbl_res_cancel;
+	new->rbl_dns_setns = rbl_res_nslist;
+	new->rbl_dns_close = rbl_res_close;
 
 	return new;
 }
@@ -290,6 +399,10 @@ rbl_close(RBL *rbl)
 {
 	assert(rbl != NULL);
 
+	if (rbl->rbl_dns_service != NULL &&
+	    rbl->rbl_dns_close != NULL)
+		(void) rbl->rbl_dns_close(rbl->rbl_dns_service);
+
 	if (rbl->rbl_free != NULL)
 		rbl->rbl_free(rbl->rbl_closure, rbl);
 	else
@@ -529,6 +642,253 @@ rbl_dns_set_query_waitreply(RBL *rbl, int (*func)(void *, void *,
 	rbl->rbl_dns_waitreply = func;
 }
 
+/*
+**  RBL_DNS_SET_NSLIST -- stores a pointer to a NS list update function
+**
+**  Parameters:
+**  	lib -- RBL library handle
+**  	func -- function to use to update NS list
+**
+**  Return value:
+**  	None.
+**
+**  Notes:
+**  	"func" should match the following prototype:
+**  		returns int
+**  		void *dns -- DNS service handle
+**  		const char *nslist -- comma-separated list of nameservers
+*/
+
+void
+rbl_dns_set_nslist(RBL *lib, int (*func)(void *, const char *))
+{
+	assert(lib != NULL);
+
+	if (func != NULL)
+		lib->rbl_dns_setns = func;
+	else
+		lib->rbl_dns_setns = rbl_res_nslist;
+}
+
+/*
+**  RBL_DNS_SET_CONFIG -- stores a pointer to a resolver configuration update
+**                        function
+**
+**  Parameters:
+**  	lib -- RBL library handle
+**  	func -- function to use to update resolver configuration
+**
+**  Return value:
+**  	None.
+**
+**  Notes:
+**  	"func" should match the following prototype:
+**  		returns int
+**  		void *dns -- DNS service handle
+**  		const char *config -- arbitrary resolver configuration data
+*/
+
+void
+rbl_dns_set_config(RBL *lib, int (*func)(void *, const char *))
+{
+	assert(lib != NULL);
+
+	lib->rbl_dns_config = func;
+}
+
+/*
+**  RBL_DNS_SET_TRUSTANCHOR -- stores a pointer to a trust anchor update
+**                             function
+**
+**  Parameters:
+**  	lib -- RBL library handle
+**  	func -- function to use to update trust anchor data
+**
+**  Return value:
+**  	None.
+**
+**  Notes:
+**  	"func" should match the following prototype:
+**  		returns int
+**  		void *dns -- DNS service handle
+**  		const char *trust -- arbitrary trust anchor data
+*/
+
+void
+rbl_dns_set_trustanchor(RBL *lib, int (*func)(void *, const char *))
+{
+	assert(lib != NULL);
+
+	lib->rbl_dns_trustanchor = func;
+}
+
+/*
+**  RBL_DNS_SET_CLOSE -- stores a pointer to a resolver shutdown function
+**
+**  Parameters:
+**  	lib -- RBL library handle
+**  	func -- function to use to initialize the resolver
+**
+**  Return value:
+**  	None.
+**
+**  Notes:
+**  	"func" should match the following prototype:
+**  		returns void
+**  		void *srv -- DNS service handle
+*/
+
+void
+rbl_dns_set_close(RBL *lib, void (*func)(void *))
+{
+	assert(lib != NULL);
+
+	if (func != NULL)
+		lib->rbl_dns_close = func;
+	else
+		lib->rbl_dns_close = rbl_res_close;
+}
+
+/*
+**  RBL_DNS_SET_INIT -- stores a pointer to a resolver init function
+**
+**  Parameters:
+**  	lib -- RBL library handle
+**  	func -- function to use to initialize the resolver
+**
+**  Return value:
+**  	None.
+**
+**  Notes:
+**  	"func" should match the following prototype:
+**  		returns int (status)
+**  		void **srv -- DNS service handle (updated)
+*/
+
+void
+rbl_dns_set_init(RBL *lib, int (*func)(void **))
+{
+	assert(lib != NULL);
+
+	lib->rbl_dns_init = func;
+}
+
+/*
+**  RBL_DNS_NSLIST -- requests update to a nameserver list
+**
+**  Parameters:
+**  	lib -- RBL library handle
+**  	nslist -- comma-separated list of nameservers to use
+**
+**  Return value:
+**  	An RBL_STAT_* constant.
+*/
+
+RBL_STAT
+rbl_dns_nslist(RBL *lib, const char *nslist)
+{
+	int status;
+
+	assert(lib != NULL);
+	assert(nslist != NULL);
+
+	if (lib->rbl_dns_setns != NULL)
+	{
+		status = lib->rbl_dns_setns(lib->rbl_dns_service, nslist);
+		if (status != 0)
+			return RBL_STAT_ERROR;
+	}
+
+	return RBL_STAT_OK;
+}
+
+/*
+**  RBL_DNS_CONFIG -- requests a change to resolver configuration
+**
+**  Parameters:
+**  	lib -- RBL library handle
+**  	config -- opaque configuration string
+**
+**  Return value:
+**  	An RBL_STAT_* constant.
+*/
+
+RBL_STAT
+rbl_dns_config(RBL *lib, const char *config)
+{
+	int status;
+
+	assert(lib != NULL);
+	assert(config != NULL);
+
+	if (lib->rbl_dns_config != NULL)
+	{
+		status = lib->rbl_dns_config(lib->rbl_dns_service, config);
+		if (status != 0)
+			return RBL_STAT_ERROR;
+	}
+
+	return RBL_STAT_OK;
+}
+
+/*
+**  RBL_DNS_TRUSTANCHOR -- requests a change to resolver trust anchor data
+**
+**  Parameters:
+**  	lib -- RBL library handle
+**  	trust -- opaque trust anchor string
+**
+**  Return value:
+**  	An RBL_STAT_* constant.
+*/
+
+RBL_STAT
+rbl_dns_trustanchor(RBL *lib, const char *trust)
+{
+	int status;
+
+	assert(lib != NULL);
+	assert(trust != NULL);
+
+	if (lib->rbl_dns_trustanchor != NULL)
+	{
+		status = lib->rbl_dns_trustanchor(lib->rbl_dns_service, trust);
+		if (status != 0)
+			return RBL_STAT_ERROR;
+	}
+
+	return RBL_STAT_OK;
+}
+
+/*
+**  RBL_DNS_INIT -- force nameserver (re)initialization
+**
+**  Parameters:
+**  	lib -- RBL library handle
+**
+**  Return value:
+**  	An RBL_STAT_* constant.
+*/
+
+RBL_STAT
+rbl_dns_init(RBL *lib)
+{
+	int status;
+
+	assert(lib != NULL);
+
+	if (lib->rbl_dns_service != NULL &&
+	    lib->rbl_dns_close != NULL)
+		lib->rbl_dns_close(lib->rbl_dns_service);
+
+	lib->rbl_dns_service = NULL;
+
+	if (lib->rbl_dns_init != NULL)
+		return lib->rbl_dns_init(&lib->rbl_dns_service);
+	else
+		return RBL_STAT_OK;
+}
+
 /*
 **  RBL_QUERY_CANCEL -- cancel an open query to the RBL
 **
@@ -602,6 +962,11 @@ rbl_query_start(RBL *rbl, u_char *query, void **qh)
 
 	memset(rq, '\0', sizeof *rq);
 
+	if (rbl->rbl_dns_service == NULL &&
+	    rbl->rbl_dns_init != NULL &&
+	    rbl->rbl_dns_init(&rbl->rbl_dns_service) != 0)
+		return RBL_STAT_DNSERROR;
+
 	status = rbl->rbl_dns_start(rbl->rbl_dns_service, T_A, rblquery,
 	                            rq->rq_buf, sizeof rq->rq_buf, &rq->rq_qh);
 
