@@ -2,7 +2,7 @@
 **  Copyright (c) 2007-2009 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  Copyright (c) 2009, 2011, 2012, The Trusted Domain Project.
+**  Copyright (c) 2009, 2011-2013, The Trusted Domain Project.
 **    All rights reserved.
 **
 */
@@ -22,6 +22,11 @@
 # include <sysexits.h>
 #endif /* ARTEST */
 
+/* libbsd if found */
+#ifdef USE_BSD_H
+# include <bsd/string.h>
+#endif /* USE_BSD_H */
+
 /* libstrl if needed */
 #ifdef USE_STRL_H
 # include <strl.h>
@@ -294,7 +299,7 @@ ares_convert(struct lookup *table, char *str)
 	for (c = 0; ; c++)
 	{
 		if (table[c].str == NULL ||
-		    strcmp(table[c].str, str) == 0)
+		    strcasecmp(table[c].str, str) == 0)
 			return table[c].code;
 	}
 
@@ -437,6 +442,9 @@ ares_parse(u_char *hdr, struct authres *ar)
 
 		  case 3:				/* method */
 			n++;
+			if (n >= MAXARESULTS)
+				return 0;
+
 			r = 0;
 
 			ar->ares_result[n - 1].result_method = ares_convert(methods,
@@ -542,7 +550,8 @@ ares_parse(u_char *hdr, struct authres *ar)
 				if (x == ARES_PTYPE_UNKNOWN)
 					return -1;
 
-				ar->ares_result[n - 1].result_ptype[r] = x;
+				if (r < MAXPROPS)
+					ar->ares_result[n - 1].result_ptype[r] = x;
 
 				prevstate = state;
 				state = 10;
@@ -561,9 +570,12 @@ ares_parse(u_char *hdr, struct authres *ar)
 			break;
 
 		  case 11:				/* property */
-			strlcpy((char *) ar->ares_result[n - 1].result_property[r],
-			        (char *) tokens[c],
-			        sizeof ar->ares_result[n - 1].result_property[r]);
+			if (r < MAXPROPS)
+			{
+				strlcpy((char *) ar->ares_result[n - 1].result_property[r],
+				        (char *) tokens[c],
+				        sizeof ar->ares_result[n - 1].result_property[r]);
+			}
 
 			prevstate = state;
 			state = 12;
@@ -581,11 +593,14 @@ ares_parse(u_char *hdr, struct authres *ar)
 			break;
 
 		  case 13:				/* value */
-			strlcat((char *) ar->ares_result[n - 1].result_value[r],
-			        (char *) tokens[c],
-			        sizeof ar->ares_result[n - 1].result_value[r]);
-			r++;
-			ar->ares_result[n - 1].result_props = r;
+			if (r < MAXPROPS)
+			{
+				strlcat((char *) ar->ares_result[n - 1].result_value[r],
+				        (char *) tokens[c],
+				        sizeof ar->ares_result[n - 1].result_value[r]);
+				r++;
+				ar->ares_result[n - 1].result_props = r;
+			}
 
 			prevstate = state;
 			state = 9;
