@@ -2,13 +2,9 @@
 **  Copyright (c) 2008 Sendmail, Inc. and its suppliers.
 **    All rights reserved.
 **
-**  Copyright (c) 2009-2012, The OpenDKIM Project.  All rights reserved.
+**  Copyright (c) 2009-2012, The Trusted Domain Project.  All rights reserved.
 */
 
-#ifndef lint
-static char opendkim_dns_c_id[] = "@(#)$Id: opendkim-dns.c,v 1.7.10.2 2010/10/28 04:28:04 cm-msk Exp $";
-#endif /* !lint */
-
 #include "build-config.h"
 
 /* system includes */
@@ -41,11 +37,10 @@ static char opendkim_dns_c_id[] = "@(#)$Id: opendkim-dns.c,v 1.7.10.2 2010/10/28
 # include <rbl.h>
 #endif /* _FFR_RBL */
 
-#ifdef USE_ARLIB
-/* libar includes */
-# include <async-resolv.h>
-# define MAXCNAMEDEPTH	3
-#endif /* USE_ARLIB */
+#ifdef _FFR_VBR
+/* libvbr includes */
+# include <vbr.h>
+#endif /* _FFR_VBR */
 
 /* opendkim includes */
 #include "opendkim-dns.h"
@@ -98,7 +93,69 @@ struct dkimf_unbound_cb_data
 	u_char *		ubd_buf;
 	const char *		ubd_strerror;
 };
+#endif /* USE_UNBOUND */
 
+/*
+**  DKIMF_DNS_TRUSTANCHOR -- advise the DKIM library of new trust anchor data
+**
+**  Parameters:
+**  	lib -- DKIM library handle
+**  	trust -- trust anchor data
+**
+**  Return value:
+**  	DKIM_DNS_SUCCESS or DKIM_DNS_ERROR
+*/
+
+int
+dkimf_dns_trustanchor(DKIM_LIB *lib, const char *trust)
+{
+	assert(lib != NULL);
+	assert(trust != NULL);
+
+	return dkim_dns_trustanchor(lib, trust);
+}
+
+/*
+**  DKIMF_DNS_SETNAMESERVERS -- advise the DKIM library of new nameservers
+**
+**  Parameters:
+**  	lib -- DKIM library handle
+**  	nslist -- nameserver list
+**
+**  Return value:
+**  	DKIM_DNS_SUCCESS or DKIM_DNS_ERROR
+*/
+
+int
+dkimf_dns_setnameservers(DKIM_LIB *lib, const char *nslist)
+{
+	assert(lib != NULL);
+	assert(nslist != NULL);
+
+	return dkim_dns_nslist(lib, nslist);
+}
+
+/*
+**  DKIMF_DNS_CONFIG -- pass arbitrary configuration to the resolver
+**
+**  Parameters:
+**  	lib -- DKIM library handle
+**  	config -- resolver configuration data
+**
+**  Return value:
+**  	DKIM_DNS_SUCCESS or DKIM_DNS_ERROR
+*/
+
+int
+dkimf_dns_config(DKIM_LIB *lib, const char *config)
+{
+	assert(lib != NULL);
+	assert(config != NULL);
+
+	return dkim_dns_config(lib, config);
+}
+
+#ifdef USE_UNBOUND
 /*
 **  DKIMF_UNBOUND_CB -- callback to handle result of DNS query
 **
@@ -336,6 +393,63 @@ dkimf_unbound_queue(struct dkimf_unbound *ub, char *name, int type,
 	return 0;
 }
 
+/*
+**  DKIMF_UB_TRUSTANCHOR -- add a trust anchor file to a libunbound context
+**
+**  Parameters:
+**  	srv -- service handle
+**  	file -- path to add
+**
+**  Return value:
+**  	0 -- success
+**  	-1 -- error
+*/
+
+int
+dkimf_ub_trustanchor(void *srv, const char *file)
+{
+	int status;
+	struct dkimf_unbound *ub;
+
+	assert(srv != NULL);
+	assert(file != NULL);
+
+	ub = srv;
+
+	status = ub_ctx_add_ta_file(ub->ub_ub, (char *) file);
+
+	return (status == 0 ? 0 : -1);
+}
+
+
+/*
+**  DKIMF_UB_CONFIG -- add a configuration file to a libunbound context
+**
+**  Parameters:
+**  	srv -- void service handle
+**  	file -- path to add
+**
+**  Return value:
+**  	0 -- success
+**  	-1 -- error
+*/
+
+int
+dkimf_ub_config(void *srv, const char *file)
+{
+	int status;
+	struct dkimf_unbound *ub;
+
+	assert(srv != NULL);
+	assert(file != NULL);
+
+	ub = srv;
+
+	status = ub_ctx_config(ub->ub_ub, (char *) file);
+
+	return (status == 0 ? 0 : -1);
+}
+
 /*
 **  DKIMF_UB_CANCEL -- function passed to libopendkim to handle cancel requests
 **
@@ -461,18 +575,18 @@ dkimf_ub_waitreply(void *srv, void *qh, struct timeval *to, size_t *bytes,
 /* =========================== PUBLIC FUNCTIONS =========================== */
 
 /*
-**  DKIMF_UNBOUND_INIT -- set up a libunbound context and other data
+**  DKIMF_UB_INIT -- set up a libunbound context and other data
 **
 **  Parameters:
 **  	ub -- unbound context (returned)
 **
 **  Return value:
-**  	0 -- success
-**  	-1 -- failure
+**  	DKIM_DNS_SUCCESS -- success
+**  	DKIM_DNS_ERROR -- failure
 */
 
 int
-dkimf_unbound_init(struct dkimf_unbound **ub)
+dkimf_ub_init(void **ub)
 {
 	struct dkimf_unbound *out;
 
@@ -480,13 +594,13 @@ dkimf_unbound_init(struct dkimf_unbound **ub)
 
 	out = (struct dkimf_unbound *) malloc(sizeof *out);
 	if (out == NULL)
-		return -1;
+		return DKIM_DNS_ERROR;
 
 	out->ub_ub = ub_ctx_create();
 	if (out->ub_ub == NULL)
 	{
 		free(out);
-		return -1;
+		return DKIM_DNS_ERROR;
 	}
 
 	/* suppress debug output */
@@ -502,11 +616,11 @@ dkimf_unbound_init(struct dkimf_unbound **ub)
 
 	*ub = out;
 
-	return 0;
+	return DKIM_DNS_SUCCESS;
 }
 
 /*
-**  DKIMF_UNBOUND_CLOSE -- shut down a libunbound context
+**  DKIMF_UB_CLOSE -- shut down a libunbound context
 **
 **  Parameters:
 **  	ub -- unbound context
@@ -516,16 +630,61 @@ dkimf_unbound_init(struct dkimf_unbound **ub)
 **  	-1 -- failure
 */
 
-int
-dkimf_unbound_close(struct dkimf_unbound *ub)
+void
+dkimf_ub_close(void *srv)
 {
-	assert(ub != NULL);
+	struct dkimf_unbound *ub;
+
+	assert(srv != NULL);
+
+	ub = srv;
 
 	ub_ctx_delete(ub->ub_ub);
 	pthread_mutex_destroy(&ub->ub_lock);
 	pthread_cond_destroy(&ub->ub_ready);
+}
 
-	return 0;
+/*
+**  DKIMF_UB_NSLIST -- set nameserver list
+**
+**  Parameters:
+**  	srv -- unbound service handle (as a void *)
+**  	nslist -- nameserver list
+**
+**  Return value:
+**  	DKIM_DNS_SUCCESS or DKIM_DNS_ERROR
+*/
+
+int
+dkimf_ub_nslist(void *srv, const char *nslist)
+{
+	char *cp;
+	char *p;
+	char *last = NULL;
+	struct dkimf_unbound *ub;
+
+	assert(srv != NULL);
+	assert(nslist != NULL);
+
+	ub = srv;
+
+	cp = strdup(nslist);
+	if (cp == NULL)
+		return DKIM_DNS_ERROR;
+
+	for (p = strtok_r(cp, ",", &last);
+	     p != NULL;
+	     p = strtok_r(NULL, ",", &last))
+	{
+		if (ub_ctx_set_fwd(ub->ub_ub, p) != 0)
+		{
+			free(cp);
+			return DKIM_DNS_ERROR;
+		}
+	}
+
+	free(cp);
+	return DKIM_DNS_SUCCESS;
 }
 
 # ifdef _FFR_RBL
@@ -541,15 +700,18 @@ dkimf_unbound_close(struct dkimf_unbound *ub)
 */
 
 int
-dkimf_rbl_unbound_setup(RBL *rbl, struct dkimf_unbound *ub)
+dkimf_rbl_unbound_setup(RBL *rbl)
 {
 	assert(rbl != NULL);
-	assert(ub != NULL);
 
-	(void) rbl_dns_set_query_service(rbl, ub);
 	(void) rbl_dns_set_query_start(rbl, dkimf_ub_query);
 	(void) rbl_dns_set_query_cancel(rbl, dkimf_ub_cancel);
 	(void) rbl_dns_set_query_waitreply(rbl, dkimf_ub_waitreply);
+	(void) rbl_dns_set_init(rbl, dkimf_ub_init);
+	(void) rbl_dns_set_close(rbl, dkimf_ub_close);
+	(void) rbl_dns_set_nslist(rbl, dkimf_ub_nslist);
+	(void) rbl_dns_set_config(rbl, dkimf_ub_config);
+	(void) rbl_dns_set_trustanchor(rbl, dkimf_ub_trustanchor);
 
 	return 0;
 }
@@ -568,15 +730,18 @@ dkimf_rbl_unbound_setup(RBL *rbl, struct dkimf_unbound *ub)
 */
 
 int
-dkimf_rep_unbound_setup(DKIM_REP dr, struct dkimf_unbound *ub)
+dkimf_rep_unbound_setup(DKIM_REP dr)
 {
 	assert(dr != NULL);
-	assert(ub != NULL);
 
-	(void) dkim_rep_dns_set_query_service(dr, ub);
 	(void) dkim_rep_dns_set_query_start(dr, dkimf_ub_query);
 	(void) dkim_rep_dns_set_query_cancel(dr, dkimf_ub_cancel);
 	(void) dkim_rep_dns_set_query_waitreply(dr, dkimf_ub_waitreply);
+	(void) dkim_rep_dns_set_init(dr, dkimf_ub_init);
+	(void) dkim_rep_dns_set_close(dr, dkimf_ub_close);
+	(void) dkim_rep_dns_set_nslist(dr, dkimf_ub_nslist);
+	(void) dkim_rep_dns_set_config(dr, dkimf_ub_config);
+	(void) dkim_rep_dns_set_trustanchor(dr, dkimf_ub_trustanchor);
 
 	return 0;
 }
@@ -594,295 +759,23 @@ dkimf_rep_unbound_setup(DKIM_REP dr, struct dkimf_unbound *ub)
 */
 
 int
-dkimf_unbound_setup(DKIM_LIB *lib, struct dkimf_unbound *ub)
+dkimf_unbound_setup(DKIM_LIB *lib)
 {
 	assert(lib != NULL);
-	assert(ub != NULL);
 
-	(void) dkim_dns_set_query_service(lib, ub);
 	(void) dkim_dns_set_query_start(lib, dkimf_ub_query);
 	(void) dkim_dns_set_query_cancel(lib, dkimf_ub_cancel);
 	(void) dkim_dns_set_query_waitreply(lib, dkimf_ub_waitreply);
+	(void) dkim_dns_set_init(lib, dkimf_ub_init);
+	(void) dkim_dns_set_close(lib, dkimf_ub_close);
+	(void) dkim_dns_set_nslist(lib, dkimf_ub_nslist);
+	(void) dkim_dns_set_config(lib, dkimf_ub_config);
+	(void) dkim_dns_set_trustanchor(lib, dkimf_ub_trustanchor);
 
 	return 0;
 }
-
-/*
-**  DKIMF_UNBOUND_ADD_TRUSTANCHOR -- add a trust anchor file to a
-**                                   libunbound context
-**
-**  Parameters:
-**  	ub -- libunbound context
-**  	file -- path to add
-**
-**  Return value:
-**  	0 -- success
-**  	-1 -- error
-*/
-
-int
-dkimf_unbound_add_trustanchor(struct dkimf_unbound *ub, char *file)
-{
-	int status;
-
-	assert(ub != NULL);
-	assert(file != NULL);
-	status = ub_ctx_add_ta_file(ub->ub_ub, file);
-
-	return (status == 0 ? 0 : -1);
-}
-
-/*
-**  DKIMF_UNBOUND_ADD_CONFFILE -- add a configuration file to a libunbound
-**                                context
-**
-**  Parameters:
-**  	ub -- libunbound context
-**  	file -- path to add
-**
-**  Return value:
-**  	0 -- success
-**  	-1 -- error
-*/
-
-int
-dkimf_unbound_add_conffile(struct dkimf_unbound *ub, char *file)
-{
-	int status;
-
-	assert(ub != NULL);
-	assert(file != NULL);
-
-	status = ub_ctx_config(ub->ub_ub, file);
-
-	return (status == 0 ? 0 : -1);
-}
-
-/*
-**  DKIMF_UNBOUND_ADD_RESOLVCONF -- tell libunbound to read a resolv.conf file
-**
-**  Parameters:
-**  	ub -- libunbound context
-**  	file -- path to read
-**
-**  Return value:
-**  	0 -- success
-**  	-1 -- error
-*/
-
-int
-dkimf_unbound_add_resolvconf(struct dkimf_unbound *ub, char *file)
-{
-	int status;
-
-	assert(ub != NULL);
-	assert(file != NULL);
-
-	status = ub_ctx_resolvconf(ub->ub_ub, file);
-
-	return (status == 0 ? 0 : -1);
-}
 #endif /* USE_UNBOUND */
 
-#ifdef USE_ARLIB
-/*
-**  DKIMF_AR_CANCEL -- function passed to libopendkim to handle cancel
-**                     requests
-**
-**  Parameters:
-**  	srv -- service handle
-**  	q -- query handle
-**
-**  Return value:
-**  	A DKIM_DNS_* constant.
-*/
-
-static int
-dkimf_ar_cancel(void *srv, void *q)
-{
-	AR_LIB ar;
-	AR_QUERY arq;
-
-	assert(srv != NULL);
-	assert(q != NULL);
-
-	ar = (AR_LIB) srv;
-	arq = (AR_QUERY) q;
-
-	(void) ar_cancelquery(ar, arq);
-
-	return DKIM_DNS_SUCCESS;
-}
-
-/*
-**  DKIMF_AR_QUERY -- function passed to libopendkim to handle new requests
-**
-**  Parameters:
-**  	srv -- service handle
-**
-**  Return value:
-**  	A DKIM_DNS_* constant.
-*/
-
-static int
-dkimf_ar_query(void *srv, int type, unsigned char *query,
-               unsigned char *buf, size_t buflen, void **qh)
-{
-	AR_LIB ar;
-	AR_QUERY q;
-
-	assert(srv != NULL);
-	assert(query != NULL);
-	assert(buf != NULL);
-	assert(qh != NULL);
-
-	ar = (AR_LIB) srv;
-
-	q = ar_addquery(ar, (char *) query, C_IN, type, MAXCNAMEDEPTH,
-	                buf, buflen, (int *) NULL, (struct timeval *) NULL);
-	if (q == NULL)
-		return DKIM_DNS_ERROR;
-
-	*qh = (void *) q;
-
-	return DKIM_DNS_SUCCESS;
-}
-
-/*
-**  DKIMF_AR_WAITREPLY -- function passed to libopendkim to handle
-**                        wait requests
-**
-**  Parameters:
-**  	srv -- service handle
-**  	q -- query handle
-**  	to -- wait timeout
-**  	bytes -- bytes (returned)
-**  	error -- error code (returned)
-**  	dnssec -- DNSSEC status (returned)
-**
-**  Return value:
-**  	A DKIM_DNS_* constant.
-*/
-
-static int
-dkimf_ar_waitreply(void *srv, void *qh, struct timeval *to, size_t *bytes,
-                   int *error, int *dnssec)
-{
-	int status;
-	size_t r;
-	AR_LIB ar;
-	AR_QUERY q;
-
-	assert(srv != NULL);
-	assert(qh != NULL);
-
-	ar = (AR_LIB) srv;
-	q = (AR_QUERY) qh;
-
-	status = ar_waitreply(ar, q, &r, to);
-	if (status == AR_STAT_SUCCESS)
-	{
-		if (dnssec != NULL)
-			*dnssec = DKIM_DNSSEC_UNKNOWN;
-		if (bytes != NULL)
-			*bytes = r;
-	}
-	else
-	{
-		if (error != NULL)
-			*error = errno;
-	}
-
-	if (status == AR_STAT_SUCCESS)
-		return DKIM_DNS_SUCCESS;
-	else if (status == AR_STAT_NOREPLY)
-		return DKIM_DNS_EXPIRED;
-	else
-		return DKIM_DNS_ERROR;
-}
-
-/* =========================== PUBLIC FUNCTIONS =========================== */
-
-# ifdef _FFR_RBL
-/*
-**  DKIMF_RBL_ARLIB_SETUP -- connect libar to librbl
-**
-**  Parameters:
-**  	rbl -- librbl handle
-**  	libar -- AR_LIB handle
-**
-**  Return value:
-**  	0 on success, -1 on failure
-*/
-
-int
-dkimf_rbl_arlib_setup(RBL *rbl, AR_LIB ar)
-{
-	assert(rbl != NULL);
-	assert(ar != NULL);
-
-	(void) rbl_dns_set_query_service(rbl, ar);
-	(void) rbl_dns_set_query_start(rbl, dkimf_ar_query);
-	(void) rbl_dns_set_query_cancel(rbl, dkimf_ar_cancel);
-	(void) rbl_dns_set_query_waitreply(rbl, dkimf_ar_waitreply);
-
-	return 0;
-}
-# endif /* _FFR_RBL */
-
-# ifdef _FFR_DKIM_REPUTATION
-/*
-**  DKIMF_REP_ARLIB_SETUP -- connect libar to libdkimrep
-**
-**  Parameters:
-**  	dr -- DKIM_REP handle
-**  	libar -- AR_LIB handle
-**
-**  Return value:
-**  	0 on success, -1 on failure
-*/
-
-int
-dkimf_rep_arlib_setup(DKIM_REP dr, AR_LIB ar)
-{
-	assert(dr != NULL);
-	assert(ar != NULL);
-
-	(void) dkim_rep_dns_set_query_service(dr, ar);
-	(void) dkim_rep_dns_set_query_start(dr, dkimf_ar_query);
-	(void) dkim_rep_dns_set_query_cancel(dr, dkimf_ar_cancel);
-	(void) dkim_rep_dns_set_query_waitreply(dr, dkimf_ar_waitreply);
-
-	return 0;
-}
-# endif /* _FFR_DKIM_REPUTATION */
-
-/*
-**  DKIMF_ARLIB_SETUP -- connect libar to libopendkim
-**
-**  Parameters:
-**  	lib -- libopendkim handle
-**  	libar -- AR_LIB handle
-**
-**  Return value:
-**  	0 on success, -1 on failure
-*/
-
-int
-dkimf_arlib_setup(DKIM_LIB *lib, AR_LIB ar)
-{
-	assert(lib != NULL);
-	assert(ar != NULL);
-
-	(void) dkim_dns_set_query_service(lib, ar);
-	(void) dkim_dns_set_query_start(lib, dkimf_ar_query);
-	(void) dkim_dns_set_query_cancel(lib, dkimf_ar_cancel);
-	(void) dkim_dns_set_query_waitreply(lib, dkimf_ar_waitreply);
-
-	return 0;
-}
-#endif /* USE_ARLIB */
-
 /*
 **  DKIMF_FILEDNS_QUERY -- function passed to libopendkim to handle new
 **                         requests
@@ -898,7 +791,6 @@ static int
 dkimf_filedns_query(void *srv, int type, unsigned char *query,
                     unsigned char *buf, size_t buflen, void **qh)
 {
-	int status;
 	struct dkimf_fquery *fq;
 	size_t qlen;
 
@@ -1153,6 +1045,43 @@ dkimf_filedns_setup(DKIM_LIB *lib, DKIMF_DB db)
 	(void) dkim_dns_set_query_start(lib, dkimf_filedns_query);
 	(void) dkim_dns_set_query_cancel(lib, dkimf_filedns_cancel);
 	(void) dkim_dns_set_query_waitreply(lib, dkimf_filedns_waitreply);
+	(void) dkim_dns_set_init(lib, NULL);
+	(void) dkim_dns_set_close(lib, NULL);
+	(void) dkim_dns_set_nslist(lib, NULL);
+	(void) dkim_dns_set_config(lib, NULL);
+	(void) dkim_dns_set_trustanchor(lib, NULL);
 
 	return 0;
 }
+
+#ifdef _FFR_VBR
+# ifdef USE_UNBOUND
+/*
+**  DKIMF_VBR_UNBOUND_SETUP -- connect libunbound to libvbr
+**
+**  Parameters:
+**  	vbr -- libvbr handle
+**  	ub -- dkimf_unbound handle to use
+**
+**  Return value:
+**  	0 on success, -1 on failure
+*/
+
+int
+dkimf_vbr_unbound_setup(VBR *vbr)
+{
+	assert(vbr != NULL);
+
+	(void) vbr_dns_set_query_start(vbr, dkimf_ub_query);
+	(void) vbr_dns_set_query_cancel(vbr, dkimf_ub_cancel);
+	(void) vbr_dns_set_query_waitreply(vbr, dkimf_ub_waitreply);
+	(void) vbr_dns_set_init(vbr, dkimf_ub_init);
+	(void) vbr_dns_set_close(vbr, dkimf_ub_close);
+	(void) vbr_dns_set_nslist(vbr, dkimf_ub_nslist);
+	(void) vbr_dns_set_config(vbr, dkimf_ub_config);
+	(void) vbr_dns_set_trustanchor(vbr, dkimf_ub_trustanchor);
+
+	return 0;
+}
+# endif /* USE_UNBOUND */
+#endif /* _FFR_VBR */
