@@ -1,5 +1,4 @@
-
--- Copyright (c) 2009, 2010, 2012, The Trusted Domain Project.
+-- Copyright (c) 2009, 2010, 2012, 2013, The Trusted Domain Project.
 --   All rights reserved.
 
 -- relaxed/simple signing test
@@ -9,7 +8,11 @@
 mt.echo("*** relaxed/simple signing test")
 
 -- setup
-sock = "unix:" .. mt.getcwd() .. "/t-sign-rs.lua.sock"
+if TESTSOCKET ~= nil then
+	sock = TESTSOCKET
+else
+	sock = "unix:" .. mt.getcwd() .. "/t-sign-rs.lua.sock"
+end
 binpath = mt.getcwd() .. "/.."
 if os.getenv("srcdir") ~= nil then
 	mt.chdir(os.getenv("srcdir"))
@@ -19,7 +22,7 @@ end
 mt.startfilter(binpath .. "/opendkim", "-x", "t-sign-rs.conf", "-p", sock)
 
 -- try to connect to it
-conn = mt.connect(sock, 40, 0.05)
+conn = mt.connect(sock, 40, 0.25)
 if conn == nil then
 	error("mt.connect() failed")
 end
