@@ -2,8 +2,7 @@
 **  Copyright (c) 2005-2009 Sendmail, Inc. and its suppliers.
 **	All rights reserved.
 **
-**  Copyright (c) 2009-2012, The Trusted Domain Project.  All rights reserved.
-**
+**  Copyright (c) 2009-2013, The Trusted Domain Project.  All rights reserved.
 */
 
 #include "build-config.h"
@@ -47,6 +46,11 @@
 # endif /* SOLARIS <= 20600 */
 #endif /* SOLARIS */
 
+/* libbsd if found */
+#ifdef USE_BSD_H
+# include <bsd/string.h>
+#endif /* USE_BSD_H */
+
 /* libstrl if needed */
 #ifdef USE_STRL_H
 # include <strl.h>
@@ -445,6 +449,7 @@ dkimf_checkip(DKIMF_DB db, struct sockaddr *ip)
 		int status;
 		int bits;
 		size_t dst_len;
+		size_t iplen;
 		char *dst;
 		struct sockaddr_in6 sin6;
 		struct in6_addr addr;
@@ -461,9 +466,28 @@ dkimf_checkip(DKIMF_DB db, struct sockaddr *ip)
 
 		inet_ntop(AF_INET6, &addr, dst, dst_len);
 		dkimf_lowercase((u_char *) dst);
+		iplen = strlen(dst);
 
 		exists = FALSE;
 
+		status = dkimf_db_get(db, ipbuf, 0, NULL, 0, &exists);
+		if (status != 0)
+			return FALSE;
+		if (exists)
+			return FALSE;
+
+		status = dkimf_db_get(db, &ipbuf[1], 0, NULL, 0,
+		                      &exists);
+		if (status != 0)
+			return FALSE;
+		if (exists)
+			return TRUE;
+
+		/* try it with square brackets */
+		memmove(&ipbuf[2], &ipbuf[1], iplen + 1);
+		ipbuf[1] = '[';
+		ipbuf[iplen + 2] = ']';
+
 		status = dkimf_db_get(db, ipbuf, 0, NULL, 0, &exists);
 		if (status != 0)
 			return FALSE;
@@ -491,6 +515,7 @@ dkimf_checkip(DKIMF_DB db, struct sockaddr *ip)
 
 			inet_ntop(AF_INET6, &addr, dst, dst_len);
 			dkimf_lowercase((u_char *) dst);
+			iplen = strlen(dst);
 
 			sz = strlcat(ipbuf, "/", sizeof ipbuf);
 			if (sz >= sizeof ipbuf)
@@ -518,6 +543,38 @@ dkimf_checkip(DKIMF_DB db, struct sockaddr *ip)
 			else if (exists)
 				return TRUE;
 
+			/* try it with square brackets */
+			memmove(&ipbuf[2], &ipbuf[1], iplen + 1);
+			ipbuf[1] = '[';
+			ipbuf[iplen + 2] = ']';
+			ipbuf[iplen + 3] = '\0';
+
+			sz = strlcat(ipbuf, "/", sizeof ipbuf);
+			if (sz >= sizeof ipbuf)
+				return FALSE;
+
+			dst = &ipbuf[sz];
+			dst_len = sizeof ipbuf - sz;
+
+			sz = snprintf(dst, dst_len, "%d", 128 - bits);
+			if (sz >= sizeof ipbuf)
+				return FALSE;
+
+			exists = FALSE;
+
+			status = dkimf_db_get(db, ipbuf, 0, NULL, 0, &exists);
+			if (status != 0)
+				return FALSE;
+			if (exists)
+				return FALSE;
+
+			status = dkimf_db_get(db, &ipbuf[1], 0, NULL, 0,
+			                      &exists);
+			if (status != 0)
+				return FALSE;
+			if (exists)
+				return TRUE;
+
 			/* flip off a bit */
 			if (bits != 128)
 			{
@@ -538,6 +595,7 @@ dkimf_checkip(DKIMF_DB db, struct sockaddr *ip)
 		int c;
 		int status;
 		int bits;
+		size_t iplen;
 		struct in_addr addr;
 		struct in_addr mask;
 		struct sockaddr_in sin;
@@ -562,6 +620,24 @@ dkimf_checkip(DKIMF_DB db, struct sockaddr *ip)
 		if (exists)
 			return TRUE;
 
+		/* try it with square brackets */
+		memmove(&ipbuf[2], &ipbuf[1], strlen(&ipbuf[1]) + 1);
+		ipbuf[1] = '[';
+		ipbuf[strlen(ipbuf)] = ']';
+
+		status = dkimf_db_get(db, ipbuf, 0, NULL, 0, &exists);
+		if (status != 0)
+			return FALSE;
+		if (exists)
+			return FALSE;
+
+		status = dkimf_db_get(db, &ipbuf[1], 0, NULL, 0,
+		                      &exists);
+		if (status != 0)
+			return FALSE;
+		if (exists)
+			return TRUE;
+
 		/* iterate over possible bitwise expressions */
 		for (bits = 32; bits >= 0; bits--)
 		{
@@ -582,6 +658,7 @@ dkimf_checkip(DKIMF_DB db, struct sockaddr *ip)
 			ipbuf[0] = '!';
 			(void) dkimf_inet_ntoa(addr, &ipbuf[1],
 			                       sizeof ipbuf - 1);
+			iplen = strlen(&ipbuf[1]);
 			c = strlen(ipbuf);
 			ipbuf[c] = '/';
 			c++;
@@ -602,12 +679,16 @@ dkimf_checkip(DKIMF_DB db, struct sockaddr *ip)
 			if (exists)
 				return TRUE;
 
-			(void) dkimf_inet_ntoa(mask, &ipbuf[c],
-			                       sizeof ipbuf - c);
-		
+			/* try it with square brackets */
+			memmove(&ipbuf[2], &ipbuf[1], strlen(&ipbuf[1]) + 1);
+			ipbuf[1] = '[';
+			ipbuf[iplen + 2] = ']';
+			ipbuf[iplen + 3] = '/';
+			snprintf(&ipbuf[iplen + 4], sizeof ipbuf - iplen - 4,
+			         "%d", bits);
+
 			exists = FALSE;
-			status = dkimf_db_get(db, ipbuf, 0, NULL, 0,
-			                      &exists);
+			status = dkimf_db_get(db, ipbuf, 0, NULL, 0, &exists);
 			if (status != 0)
 				return FALSE;
 			if (exists)
@@ -932,8 +1013,13 @@ dkimf_mkpath(char *path, size_t pathlen, char *root, char *file)
 	}
 	else if (root[0] == '\0')			/* no root, use cwd */
 	{
-		(void) getcwd(path, pathlen);
-		strlcat(path, "/", pathlen);
+		char *p;
+
+		p = getcwd(path, pathlen);
+		if (p == NULL)
+			strlcpy(path, "./", pathlen);
+		else
+			strlcat(path, "/", pathlen);
 		strlcat(path, file, pathlen);
 	}
 	else						/* use root */
