@@ -1,5 +1,5 @@
 /*
-**  Copyright (c) 2010-2012, The Trusted Domain Project.  All rights reserved.
+**  Copyright (c) 2010-2013, The Trusted Domain Project.  All rights reserved.
 */
 
 #include "build-config.h"
@@ -23,6 +23,11 @@
 /* OpenDKIM includes */
 #include "stats.h"
 
+/* libbsd if found */
+#ifdef USE_BSD_H
+# include <bsd/string.h>
+#endif /* USE_BSD_H */
+
 /* libstrl if needed */
 #ifdef USE_STRL_H
 # include <strl.h>
